@@ -1,6 +1,6 @@
 /* gui.h
  *
- * Copyright 2002-2003 Vesa Halttunen
+ * Copyright 2002-2004 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -33,8 +33,7 @@ struct gui {
   GladeXML *xml;
   GtkWidget *window_main;
   GtkWidget *window_about;
-  GtkListStore *store_sectionlist, *store_playseq;
-  GtkListStore *store_blocklist, *store_messagelist;
+  unsigned int *numplayseqs, *numblocks;
 
   /* FIX: These should not be here. Current Message number */
   int messagelist_message;
@@ -61,11 +60,10 @@ enum {
   GUI_REFRESH_INFO_SHIFT,
   GUI_REFRESH_TIMER_SHIFT,
   GUI_REFRESH_BLOCKLIST_SHIFT,
-  GUI_REFRESH_BLOCKLIST_INFO_SHIFT,
   GUI_REFRESH_PLAYSEQ_SHIFT,
-  GUI_REFRESH_PLAYSEQ_INFO_SHIFT,
+  GUI_REFRESH_PLAYSEQ_RECREATE_SHIFT,
+  GUI_REFRESH_PLAYSEQLIST_SHIFT,
   GUI_REFRESH_SECTIONLIST_SHIFT,
-  GUI_REFRESH_SECTIONLIST_BUTTONS_SHIFT,
   GUI_REFRESH_MESSAGELIST_SHIFT,
   GUI_REFRESH_TRACKVOLUMES_SHIFT,
   GUI_REFRESH_SONG_SHIFT,
@@ -80,18 +78,50 @@ enum {
   GUI_REFRESH_INFO = (1 << GUI_REFRESH_INFO_SHIFT),
   GUI_REFRESH_TIMER = (1 << GUI_REFRESH_TIMER_SHIFT),
   GUI_REFRESH_BLOCKLIST = (1 << GUI_REFRESH_BLOCKLIST_SHIFT),
-  GUI_REFRESH_BLOCKLIST_INFO = (1 << GUI_REFRESH_BLOCKLIST_INFO_SHIFT),
   GUI_REFRESH_PLAYSEQ = (1 << GUI_REFRESH_PLAYSEQ_SHIFT),
-  GUI_REFRESH_PLAYSEQ_INFO = (1 << GUI_REFRESH_PLAYSEQ_INFO_SHIFT),
+  GUI_REFRESH_PLAYSEQ_RECREATE = (1 << GUI_REFRESH_PLAYSEQ_RECREATE_SHIFT),
+  GUI_REFRESH_PLAYSEQLIST = (1 << GUI_REFRESH_PLAYSEQLIST_SHIFT),
   GUI_REFRESH_SECTIONLIST = (1 << GUI_REFRESH_SECTIONLIST_SHIFT),
-  GUI_REFRESH_SECTIONLIST_BUTTONS = (1 << GUI_REFRESH_SECTIONLIST_BUTTONS_SHIFT),
   GUI_REFRESH_MESSAGELIST = (1 << GUI_REFRESH_MESSAGELIST_SHIFT),
   GUI_REFRESH_TRACKVOLUMES = (1 << GUI_REFRESH_TRACKVOLUMES_SHIFT),
   GUI_REFRESH_SONG = (1 << GUI_REFRESH_SONG_SHIFT),
   GUI_REFRESH_TRACKER = (1 << GUI_REFRESH_TRACKER_SHIFT)
 };
 
-struct gui *gui_open(struct editor *, char *);
+enum {
+  GUI_SECTIONLIST_COLUMN_SECTION,
+  GUI_SECTIONLIST_COLUMN_PLAYSEQ_NUMBER,
+  GUI_SECTIONLIST_COLUMN_PLAYSEQ_NAME
+};
+
+enum {
+  GUI_PLAYSEQLIST_COLUMN_NUMBER,
+  GUI_PLAYSEQLIST_COLUMN_NAME
+};
+
+enum {
+  GUI_PLAYSEQ_COLUMN_POSITION,
+  GUI_PLAYSEQ_COLUMN_BLOCK_NUMBER,
+  GUI_PLAYSEQ_COLUMN_BLOCK_NAME
+};
+
+enum {
+  GUI_BLOCKLIST_COLUMN_NUMBER,
+  GUI_BLOCKLIST_COLUMN_NAME,
+  GUI_BLOCKLIST_COLUMN_TRACKS,
+  GUI_BLOCKLIST_COLUMN_LENGTH,
+  GUI_BLOCKLIST_COLUMN_COMMAND_PAGES
+};
+
+enum {
+  GUI_MESSAGELIST_COLUMN_NUMBER,
+  GUI_MESSAGELIST_COLUMN_NAME,
+  GUI_MESSAGELIST_COLUMN_LENGTH,
+  GUI_MESSAGELIST_COLUMN_AUTOSEND
+};
+
+/* Opening and closing the GUI */
+struct gui *gui_open(struct editor *);
 void gui_close(struct gui *);
 
 /* For setting elements to refresh from the player thread only */
@@ -101,24 +131,21 @@ void gui_set_refresh(struct gui *, unsigned int);
 void gui_set_callback_refresh(struct gui *, unsigned int);
 void gui_refresh(struct gui *);
 void gui_refresh_all(struct gui *);
-void gui_instrument_refresh(struct gui *, gboolean);
+void gui_instrument_refresh(struct gui *);
 void gui_tempo_refresh(struct gui *);
 void gui_window_title_refresh(struct gui *);
 void gui_statusbar_refresh(struct gui *);
 void gui_info_refresh(struct gui *);
 void gui_timer_refresh(struct gui *, unsigned int);
 void gui_blocklist_refresh(struct gui *, gboolean);
-void gui_blocklist_info_refresh(struct gui *);
 void gui_playseq_refresh(struct gui *, gboolean);
-void gui_playseq_info_refresh(struct gui *);
+void gui_playseqlist_refresh(struct gui *, gboolean);
 void gui_sectionlist_refresh(struct gui *, gboolean);
 void gui_sectionlist_buttons_refresh(struct gui *);
 void gui_messagelist_refresh(struct gui *, gboolean);
 void gui_trackvolumes_refresh(struct gui *);
 void gui_preferences_refresh(struct gui *);
-/* Refresh the tracker */
 void gui_tracker_refresh(struct gui *);
-/* Refresh the song properties window widgets */
 void gui_song_refresh(struct gui *);
 
 Tracker *gui_get_tracker(struct gui *);
