@@ -1,6 +1,6 @@
 /* conversion.c
  *
- * Copyright 2002-2003 Vesa Halttunen
+ * Copyright 2002-2004 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -20,7 +20,7 @@
  */
 
 #include <string.h>
-#include <glib.h>       
+#include <glib.h>
 #include "conversion.h"
 
 /* Converts an MMD2 module to a song */
@@ -34,7 +34,7 @@ struct song *MMD2_convert(struct MMD2 *mmd)
     fprintf(stderr, "MMD2_convert() called with null MMD2\n");
     return NULL;
   }
-  
+
   /* Create a table for converting MMD commands; values need special handling */
   for (i = 0; i < 256; i++)
     commandconversion[i] = i;
@@ -73,7 +73,7 @@ struct song *MMD2_convert(struct MMD2 *mmd)
 
   song = (struct song *)calloc(1, sizeof(struct song));
   if (mmd->expdata != NULL && mmd->expdata->songname != NULL) {
-    gchar *name=g_convert(mmd->expdata->songname, strlen(mmd->expdata->songname), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+    gchar *name = g_convert(mmd->expdata->songname, strlen(mmd->expdata->songname), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
     song->name = strdup(name);
     g_free(name);
   } else
@@ -95,7 +95,7 @@ struct song *MMD2_convert(struct MMD2 *mmd)
       struct PlaySeq *PlaySeq = mmd->song->playseqtable[i];
       song->playseqs[i] = playseq;
       if (PlaySeq->name != NULL) {
-	gchar *name=g_convert(PlaySeq->name, strlen(PlaySeq->name), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+	gchar *name = g_convert(PlaySeq->name, strlen(PlaySeq->name), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
 	playseq->name = strdup(name);
 	g_free(name);
       }
@@ -119,7 +119,7 @@ struct song *MMD2_convert(struct MMD2 *mmd)
       song->blocks[i] = block;
 
       if (MMD1Block->info != NULL && MMD1Block->info->blockname != NULL) {
-	gchar *name=g_convert(MMD1Block->info->blockname, strlen(MMD1Block->info->blockname), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+	gchar *name = g_convert(MMD1Block->info->blockname, strlen(MMD1Block->info->blockname), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
 	block->name = strdup(name);
 	g_free(name);
       }
@@ -135,11 +135,11 @@ struct song *MMD2_convert(struct MMD2 *mmd)
 	  MMD2_commands_convert(&(block->commands[(j * block->tracks + k) * 2]), mmd->expdata->mmdcmd3x);
 	}
 
-      if (MMD1Block->info != NULL && MMD1Block->info->pagetable!=NULL) {
+      if (MMD1Block->info != NULL && MMD1Block->info->pagetable != NULL) {
 	for (j = 0; j < MMD1Block->info->pagetable->num_pages; j++) {
 	  for (k = 0; k < (MMD1Block->lines + 1); k++)
 	    for (l = 0; l < MMD1Block->numtracks; l++) {
-              unsigned char *commands=(unsigned char *)MMD1Block->info->pagetable->page[j];
+	      unsigned char *commands = (unsigned char *)MMD1Block->info->pagetable->page[j];
 
 	      block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2] = commandconversion[commands[(k * MMD1Block->numtracks + l) * 2]];
 	      block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2 + 1] = commands[(k * MMD1Block->numtracks + l) * 2 + 1];
@@ -156,7 +156,7 @@ struct song *MMD2_convert(struct MMD2 *mmd)
     for (i = 0; i < song->numinstruments; i++) {
       song->instruments[i] = instrument_alloc();
       if (mmd->expdata) {
-	gchar *name=g_convert(mmd->expdata->iinfo[i].name, strlen(mmd->expdata->iinfo[i].name), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+	gchar *name = g_convert(mmd->expdata->iinfo[i].name, strlen(mmd->expdata->iinfo[i].name), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
 	song->instruments[i]->name = strdup(name);
 	song->instruments[i]->hold = mmd->expdata->exp_smp[i].hold;
 	g_free(name);
@@ -199,7 +199,7 @@ struct song *MMD2_convert(struct MMD2 *mmd)
 	memcpy(song->messages[i]->data, dumps[i]->data, song->messages[i]->length);
 
 	if (dumps[i]->ext_len >= 20) {
-	  gchar *name=g_convert(dumps[i]->name, strlen(dumps[i]->name), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+	  gchar *name = g_convert(dumps[i]->name, strlen(dumps[i]->name), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
 	  song->messages[i]->name = strdup(name);
 	  g_free(name);
 	}
@@ -313,7 +313,8 @@ struct MMD2 *song_convert(struct song *song)
   struct MMD2 *mmd;
   int i, j, k, l;
   unsigned char commandconversion[256];
-  
+  unsigned int controllers_used = 0;
+
   if (song == NULL) {
     fprintf(stderr, "song_convert() called with null song\n");
     return NULL;
@@ -321,7 +322,7 @@ struct MMD2 *song_convert(struct song *song)
 
   /* Create a table for converting song commands; values need special handling */
   for (i = 0; i < 256; i++)
-    commandconversion[i] = i; 
+    commandconversion[i] = i;
   commandconversion[COMMAND_PITCH_WHEEL] = 0x03;
   commandconversion[COMMAND_END_BLOCK] = 0x1d;
   commandconversion[COMMAND_PLAYSEQ_POSITION] = 0x0b;
@@ -337,19 +338,28 @@ struct MMD2 *song_convert(struct song *song)
   mmd = (struct MMD2 *)calloc(1, sizeof(struct MMD2));
   mmd->id = ID_MMD2;
   mmd->song = (struct MMD2song *)(calloc(1, sizeof(struct MMD2song)));
-  mmd->blockarr=(struct MMD1Block **)calloc(song->numblocks, sizeof(struct MMD1Block *));
+  mmd->blockarr = (struct MMD1Block **)calloc(song->numblocks, sizeof(struct MMD1Block *));
   mmd->expdata = (struct MMD0exp *)calloc(1, sizeof(struct MMD0exp));
 
+  /* These are here to match default OctaMED settings */
+  mmd->mmdflags = MMD_LOADTOFASTMEM;
+  mmd->actplayline = -1;
+  mmd->song->flags3 = FLAG3_FREEPAN;
+  mmd->song->voladj = 100;
+  mmd->song->channels = 4;
+  mmd->song->mix_echodepth = 2;
+  mmd->song->mix_echolen = 150;
+  
   /* Song name */
-  if(song->name!=NULL) {
-    gchar *name=g_convert(song->name, strlen(song->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
-    mmd->expdata->songname=strdup(name);
-    mmd->expdata->songnamelen=strlen(mmd->expdata->songname)+1;
+  if (song->name != NULL) {
+    gchar *name = g_convert(song->name, strlen(song->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
+    mmd->expdata->songname = strdup(name);
+    mmd->expdata->songnamelen = strlen(mmd->expdata->songname) + 1;
     g_free(name);
   }
   /* Tempo */
   mmd->song->deftempo = song->tempo;
-  mmd->song->flags2 = (4-1)|FLAG2_BPM;
+  mmd->song->flags2 = (4 - 1) | FLAG2_BPM;
   mmd->song->tempo2 = song->ticksperline;
   /* Sections */
   mmd->song->songlen = song->numsections;
@@ -361,12 +371,12 @@ struct MMD2 *song_convert(struct song *song)
   mmd->song->playseqtable = (struct PlaySeq **)calloc(mmd->song->numpseqs, sizeof(struct PlaySeq *));
   for (i = 0; i < song->numplayseqs; i++) {
     struct playseq *playseq = song->playseqs[i];
-    struct PlaySeq *PlaySeq = (struct PlaySeq *)calloc(1, sizeof(struct PlaySeq)+playseq->length*sizeof(unsigned short));
+    struct PlaySeq *PlaySeq = (struct PlaySeq *)calloc(1, sizeof(struct PlaySeq) + playseq->length * sizeof(unsigned short));
     mmd->song->playseqtable[i] = PlaySeq;
-    if (playseq->name!=NULL) {
-      gchar *name=g_convert(playseq->name, strlen(playseq->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
+    if (playseq->name != NULL) {
+      gchar *name = g_convert(playseq->name, strlen(playseq->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
       strncpy(PlaySeq->name, name, 32);
-      PlaySeq->name[31]=0;
+      PlaySeq->name[31] = 0;
       g_free(name);
     }
     PlaySeq->length = playseq->length;
@@ -377,21 +387,24 @@ struct MMD2 *song_convert(struct song *song)
   mmd->song->numblocks = song->numblocks;
   mmd->blockarr = (struct MMD1Block **)calloc(mmd->song->numblocks, sizeof(struct MMD1Block *));
   for (i = 0; i < mmd->song->numblocks; i++) {
-    struct block *block=song->blocks[i];
-    struct MMD1Block *MMD1Block=(struct MMD1Block *)calloc(1, sizeof(struct MMD1Block)+4*block->tracks*block->length);
-    unsigned char *notes=(unsigned char *)MMD1Block + sizeof(struct MMD1Block);
-    MMD1Block->info=(struct BlockInfo *)calloc(1, sizeof(struct BlockInfo));
-    if(block->commandpages>1) {
-      MMD1Block->info->pagetable=(struct BlockCmdPageTable *)calloc(1, sizeof(struct BlockCmdPageTable)+(block->commandpages - 1)*sizeof(unsigned short *));
-      MMD1Block->info->pagetable->num_pages=block->commandpages - 1;
+    struct block *block = song->blocks[i];
+    struct MMD1Block *MMD1Block = (struct MMD1Block *)calloc(1, sizeof(struct MMD1Block) + 4 * block->tracks * block->length);
+    unsigned char *notes = (unsigned char *)MMD1Block + sizeof(struct MMD1Block);
+
+    if (block->commandpages > 1 || block->name != NULL)
+      MMD1Block->info = (struct BlockInfo *)calloc(1, sizeof(struct BlockInfo));
+
+    if (block->commandpages > 1) {
+      MMD1Block->info->pagetable = (struct BlockCmdPageTable *)calloc(1, sizeof(struct BlockCmdPageTable) + (block->commandpages - 1) * sizeof(unsigned short *));
+      MMD1Block->info->pagetable->num_pages = block->commandpages - 1;
     }
-    
+
     mmd->blockarr[i] = MMD1Block;
 
     if (block->name != NULL) {
-      gchar *name=g_convert(block->name, strlen(block->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
+      gchar *name = g_convert(block->name, strlen(block->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
       MMD1Block->info->blockname = strdup(name);
-      MMD1Block->info->blocknamelen=strlen(MMD1Block->info->blockname);
+      MMD1Block->info->blocknamelen = strlen(MMD1Block->info->blockname);
       g_free(name);
     }
     MMD1Block->numtracks = block->tracks;
@@ -399,40 +412,69 @@ struct MMD2 *song_convert(struct song *song)
 
     for (j = 0; j < (MMD1Block->lines + 1); j++)
       for (k = 0; k < MMD1Block->numtracks; k++) {
-        notes[(j * block->tracks + k) * 4] = block->notes[(j * block->tracks + k) * 2];
-        notes[(j * block->tracks + k) * 4 + 1] = block->notes[(j * block->tracks + k) * 2 + 1];
-        notes[(j * block->tracks + k) * 4 + 2] = commandconversion[block->commands[(j * block->tracks + k) * 2]];
-        notes[(j * block->tracks + k) * 4 + 3] = block->commands[(j * block->tracks + k) * 2 + 1];
-        /* Command values may need special handling */
-        song_commands_convert(&(notes[(j * block->tracks + k) * 4 + 2]));
+	notes[(j * block->tracks + k) * 4] = block->notes[(j * block->tracks + k) * 2];
+	notes[(j * block->tracks + k) * 4 + 1] = block->notes[(j * block->tracks + k) * 2 + 1];
+	notes[(j * block->tracks + k) * 4 + 2] = commandconversion[block->commands[(j * block->tracks + k) * 2]];
+	notes[(j * block->tracks + k) * 4 + 3] = block->commands[(j * block->tracks + k) * 2 + 1];
+
+	/* Convert MIDI controllers to MMD3x commands */
+	if (notes[(j * block->tracks + k) * 4 + 2] >= 0x80) {
+	  if (mmd->expdata->mmdcmd3x == NULL) {
+	    mmd->expdata->mmdcmd3x = (struct MMDMIDICmd3x *)calloc(1, sizeof(struct MMDMIDICmd3x));
+	    mmd->expdata->mmdcmd3x->num_of_settings = 15;
+	    mmd->expdata->mmdcmd3x->ctrlr_types = (unsigned char *)calloc(15, sizeof(unsigned char));
+	    mmd->expdata->mmdcmd3x->ctrlr_numbers = (unsigned short *)calloc(15, sizeof(unsigned short));
+	  }
+
+	  /* Check if the controller already has a MMD3x command */
+	  for (l = 0; l < controllers_used; l++)
+	    if (mmd->expdata->mmdcmd3x->ctrlr_numbers[l] == notes[(j * block->tracks + k) * 4 + 2] - 0x80)
+	      break;
+
+	  if (l == controllers_used) {
+	    if (controllers_used < 15) {
+	      mmd->expdata->mmdcmd3x->ctrlr_numbers[l] = notes[(j * block->tracks + k) * 4 + 2] - 0x80;
+	      controllers_used++;
+	      notes[(j * block->tracks + k) * 4 + 2] = 0x31 + l;
+	    }
+	  } else
+	    notes[(j * block->tracks + k) * 4 + 2] = 0x31 + l;
+	}
+
+	/* Command values may need special handling */
+	song_commands_convert(&(notes[(j * block->tracks + k) * 4 + 2]));
       }
 
-    if(MMD1Block->info->pagetable!=NULL) {
+    if (MMD1Block->info != NULL && MMD1Block->info->pagetable != NULL) {
       for (j = 0; j < MMD1Block->info->pagetable->num_pages; j++) {
-        MMD1Block->info->pagetable->page[j] = (unsigned short *)calloc(MMD1Block->numtracks * (MMD1Block->lines + 1) * 2, sizeof(unsigned short));
-        for (k = 0; k < (MMD1Block->lines + 1); k++)
-          for (l = 0; l < MMD1Block->numtracks; l++) {
-            unsigned char *commands=(unsigned char *)MMD1Block->info->pagetable->page[j];
-            commands[(k * MMD1Block->numtracks + l) * 2] = commandconversion[block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2]];
-            commands[(k * MMD1Block->numtracks + l) * 2 + 1] = block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2 + 1];
-            /* Command values may need special handling */
-            song_commands_convert(&(commands[(k * MMD1Block->numtracks + l) * 2]));
-          }
+	MMD1Block->info->pagetable->page[j] = (unsigned short *)calloc(MMD1Block->numtracks * (MMD1Block->lines + 1) * 2, sizeof(unsigned short));
+	for (k = 0; k < (MMD1Block->lines + 1); k++)
+	  for (l = 0; l < MMD1Block->numtracks; l++) {
+	    unsigned char *commands = (unsigned char *)MMD1Block->info->pagetable->page[j];
+	    commands[(k * MMD1Block->numtracks + l) * 2] = commandconversion[block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2]];
+	    commands[(k * MMD1Block->numtracks + l) * 2 + 1] = block->commands[(j + 1) * 2 * block->tracks * block->length + (k * block->tracks + l) * 2 + 1];
+	    /* Command values may need special handling */
+	    song_commands_convert(&(commands[(k * MMD1Block->numtracks + l) * 2]));
+	  }
       }
     }
   }
+
   /* Instruments */
   mmd->song->numsamples = song->numinstruments;
-  mmd->expdata->s_ext_entries=mmd->song->numsamples;
-  mmd->expdata->s_ext_entrsz=sizeof(struct InstrExt);
-  mmd->expdata->i_ext_entries=mmd->song->numsamples;
-  mmd->expdata->i_ext_entrsz=sizeof(struct MMDInstrInfo);
-  mmd->expdata->exp_smp=(struct InstrExt *)calloc(mmd->expdata->s_ext_entries, sizeof(struct InstrExt));
-  mmd->expdata->iinfo=(struct MMDInstrInfo *)calloc(mmd->expdata->i_ext_entries, sizeof(struct MMDInstrInfo));
+  mmd->smplarr = (struct InstrHdr **)calloc(mmd->song->numsamples, sizeof(struct InstrHdr *));
+  mmd->expdata->s_ext_entries = mmd->song->numsamples;
+  /* I have no idea why this is not sizeof(struct InstrExt); like it should be */
+  mmd->expdata->s_ext_entrsz = 18;
+  mmd->expdata->i_ext_entries = mmd->song->numsamples;
+  /* I have no idea why this is not sizeof(struct MMDInstrInfo); like it should be */
+  mmd->expdata->i_ext_entrsz = 42;
+  mmd->expdata->exp_smp = (struct InstrExt *)calloc(mmd->expdata->s_ext_entries, sizeof(struct InstrExt));
+  mmd->expdata->iinfo = (struct MMDInstrInfo *)calloc(mmd->expdata->i_ext_entries, sizeof(struct MMDInstrInfo));
   for (i = 0; i < mmd->song->numsamples; i++) {
-    gchar *name=g_convert(song->instruments[i]->name, strlen(song->instruments[i]->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
+    gchar *name = g_convert(song->instruments[i]->name, strlen(song->instruments[i]->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
     strncpy(mmd->expdata->iinfo[i].name, name, 40);
-    mmd->expdata->iinfo[i].name[39]=0;
+    mmd->expdata->iinfo[i].name[39] = 0;
     mmd->expdata->exp_smp[i].hold = song->instruments[i]->hold;
     mmd->song->sample[i].midich = song->instruments[i]->midichannel + 1;
     mmd->song->sample[i].midipreset = song->instruments[i]->midipreset;
@@ -450,22 +492,22 @@ struct MMD2 *song_convert(struct song *song)
   mmd->song->mastervol = (song->mastervolume + 1) / 2;
 
   /* SysEX dumps */
-  mmd->expdata->dumps=(struct MMDDumpData *)calloc(1, sizeof(struct MMDDumpData)+song->nummessages*sizeof(struct MMDDump *));
-  mmd->expdata->dumps->numdumps = song->nummessages;
-  struct MMDDump **dumps = (struct MMDDump **)((char *)mmd->expdata->dumps + sizeof(struct MMDDumpData));
-      
-  for (i = 0; i < mmd->expdata->dumps->numdumps; i++) {
-    dumps[i] = (struct MMDDump *)calloc(1, sizeof(struct MMDDump));
-    dumps[i]->length = song->messages[i]->length;
-    dumps[i]->data = (unsigned char *)calloc(dumps[i]->length, sizeof(unsigned char));
-    memcpy(dumps[i]->data, song->messages[i]->data, dumps[i]->length);
-      
-    if(song->messages[i]->name!=NULL) {
-      gchar *name=g_convert(song->messages[i]->name, strlen(song->messages[i]->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
-      strncpy(dumps[i]->name, name, 20);
-      dumps[i]->name[19]=0;
-      dumps[i]->ext_len=strlen(name);
-      g_free(name);
+  if (song->nummessages > 0) {
+    mmd->expdata->dumps = (struct MMDDumpData *)calloc(1, sizeof(struct MMDDumpData) + song->nummessages * sizeof(struct MMDDump *));
+    mmd->expdata->dumps->numdumps = song->nummessages;
+    struct MMDDump **dumps = (struct MMDDump **)((char *)mmd->expdata->dumps + sizeof(struct MMDDumpData));
+    for (i = 0; i < mmd->expdata->dumps->numdumps; i++) {
+      dumps[i] = (struct MMDDump *)calloc(1, sizeof(struct MMDDump));
+      dumps[i]->length = song->messages[i]->length;
+      dumps[i]->data = (unsigned char *)calloc(dumps[i]->length, sizeof(unsigned char));
+      memcpy(dumps[i]->data, song->messages[i]->data, dumps[i]->length);
+      if (song->messages[i]->name != NULL) {
+	gchar *name = g_convert(song->messages[i]->name, strlen(song->messages[i]->name), "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
+	strncpy(dumps[i]->name, name, 20);
+	dumps[i]->name[19] = 0;
+	dumps[i]->ext_len = strlen(name);
+	g_free(name);
+      }
     }
   }
 
@@ -478,13 +520,30 @@ void song_commands_convert(unsigned char *command)
   switch (command[0]) {
   case 0x0c:
     /* Fix volume range to 0..64 */
-    command[1]=(command[1]+1)/2;
+    command[1] = (command[1] + 1) / 2;
     break;
   case 0x20:
     /* Delay */
     command[0] = 0x1f;
-    command[1]<<=4;
-    break;  
+    command[1] <<= 4;
+    break;
+  case 0x31:
+  case 0x32:
+  case 0x33:
+  case 0x34:
+  case 0x35:
+  case 0x36:
+  case 0x37:
+  case 0x38:
+  case 0x39:
+  case 0x3a:
+  case 0x3b:
+  case 0x3c:
+  case 0x3d:
+  case 0x3e:
+  case 0x3f:
+    /* MMD3x: OctaMED doesn't support value interpolation */
+    command[1] &= 0x7f;
   default:
     break;
   }
