@@ -1,6 +1,6 @@
 /* mmd.c
  *
- * Copyright 2002-2003 Vesa Halttunen
+ * Copyright 2002-2004 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -232,7 +232,7 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
     putbyte(data + offsets[0] + i, mmd->song->trackpans[i]);
   }
   offsets[0] += ((mmd->song->numtracks + 1) & 0xfffffffe);
-
+  
   /* MMD2Song */
   putlong(data + 8, offsets[0]);
   for (i = 0; i < 63; i++) {
@@ -415,6 +415,8 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
       memcpy(data + offsets[0], mmd->expdata->songname, mmd->expdata->songnamelen);
       offsets[0] += (mmd->expdata->songnamelen + 1) & 0xfffffffe;
     }
+    /* Dumps */
+    offsets[7] = 0;
     if (mmd->expdata->dumps != NULL) {
       struct MMDDump **dumps = (struct MMDDump **)((char *)mmd->expdata->dumps + sizeof(struct MMDDumpData));
 
@@ -439,6 +441,7 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
       }
       offsets[0] += sizeof(struct MMDDumpData) + mmd->expdata->dumps->numdumps * sizeof(int);
     }
+    /* MMDArexx */
     offsets[8] = 0;
     if (mmd->expdata->mmdrexx != NULL) {
       if (mmd->expdata->mmdrexx->trigcmd != NULL) {
@@ -490,6 +493,7 @@ void MMD2_save(struct MMD2 *mmd, char *filename)
 	offsets[0] += sizeof(struct MMDARexx);
       }
     }
+    /* MMDCmd3x */
     offsets[9] = 0;
     if (mmd->expdata->mmdcmd3x != NULL) {
       putlong(data + offsets[0] + ((mmd->expdata->mmdcmd3x->num_of_settings + 1) & 0xfffffffe) + mmd->expdata->mmdcmd3x->num_of_settings * sizeof(unsigned short) + 4, offsets[0]);
@@ -809,8 +813,10 @@ struct MMD0exp *MMD0exp_parse(unsigned char *base, int offset)
     }
   }
   mmd0exp->annolen = getlong(data + 16);
-  mmd0exp->annotxt = (unsigned char *)calloc(mmd0exp->annolen, sizeof(unsigned char));
-  memcpy(mmd0exp->annotxt, base + getlong(data + 12), mmd0exp->annolen);
+  if(getlong(data + 12) != 0) {
+    mmd0exp->annotxt = (unsigned char *)calloc(mmd0exp->annolen, sizeof(unsigned char));
+    memcpy(mmd0exp->annotxt, base + getlong(data + 12), mmd0exp->annolen);
+  }
   /* annotext */
   mmd0exp->i_ext_entries = getword(data + 24);
   mmd0exp->i_ext_entrsz = getword(data + 26);
@@ -870,7 +876,7 @@ struct MMD0exp *MMD0exp_parse(unsigned char *base, int offset)
 	memcpy(dumps[i]->name, data3 + 10, 20);
     }
   }
-
+  
   /* mmdinfo */
   if (getlong(data + 56) != 0) {
     data2 = base + getlong(data + 56);
