@@ -1,6 +1,6 @@
 /* player.c
  *
- * Copyright 2002-2003 Vesa Halttunen
+ * Copyright 2002-2004 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -475,7 +475,7 @@ static void player_thread(struct player *player)
 	/* Use previous instrument if none defined */
 	if (instrument == 0)
 	  instrument = player->trackstatus[i * TRACK_STATUS_NUMVALUES + TRACK_STATUS_INSTRUMENT] + 1;
-	
+
 	/* Play note if instrument is defined */
 	if (instrument != 0)
 	  player_play_note(player, instrument - 1, note, volume, i);
@@ -483,16 +483,16 @@ static void player_thread(struct player *player)
 	/* If no hold value was defined use the instrument's hold value */
 	if (hold == -1 && instrument <= song->numinstruments && song->instruments[instrument - 1] != NULL)
 	  hold = song->instruments[instrument - 1]->hold;
-	
+
 	if (hold == 0)
 	  player->trackstatus[i * TRACK_STATUS_NUMVALUES + TRACK_STATUS_HOLD] = -1;
 	else
 	  player->trackstatus[i * TRACK_STATUS_NUMVALUES + TRACK_STATUS_HOLD] = hold;
       }
-      
+
       /* First tick, no note but instrument defined? */
       if (player->tick == 0 && note == 0 && instrument > 0 && player->trackstatus[i * TRACK_STATUS_NUMVALUES + TRACK_STATUS_HOLD] >= 0)
-        player->trackstatus[i * TRACK_STATUS_NUMVALUES + TRACK_STATUS_HOLD] += song->instruments[instrument - 1]->hold;
+	player->trackstatus[i * TRACK_STATUS_NUMVALUES + TRACK_STATUS_HOLD] += song->instruments[instrument - 1]->hold;
     }
 
     /* Decrement hold times of notes and stop notes that should be stopped */
@@ -577,6 +577,22 @@ static void player_thread(struct player *player)
     g_mutex_unlock(player->mutex);
   }
 
+  /* Calculate how long the song has been playing */
+  gettimeofday(&now, NULL);
+  now.tv_sec -= player->playingstarted.tv_sec;
+  if (now.tv_usec >= player->playingstarted.tv_usec)
+    now.tv_usec -= player->playingstarted.tv_usec;
+  else {
+    now.tv_usec += 1000000 - player->playingstarted.tv_usec;
+    now.tv_sec--;
+  }
+  player->playedsofar.tv_sec += now.tv_sec;
+  player->playedsofar.tv_usec += now.tv_usec;
+  if(player->playedsofar.tv_usec > 1000000) {
+    player->playedsofar.tv_sec++;
+    player->playedsofar.tv_usec -= 1000000;
+  }
+    
   if (player->sched == SCHED_RTC)
     ioctl(player->rtc, RTC_PIE_OFF, 0);
 
@@ -608,7 +624,7 @@ struct player *player_open(struct song *song, struct editor *editor)
   player->editor = editor;
   player->midi = midi_open(gconf_client_get_string(editor->gconfclient, "/apps/tutka/mididevice", NULL));
   player->sched = gconf_client_get_int(editor->gconfclient, "/apps/tutka/scheduler", NULL);
-  if(player->sched == SCHED_RTC) {
+  if (player->sched == SCHED_RTC) {
     player->rtc = open("/dev/rtc", O_RDONLY);
     if (player->rtc != -1)
       player->sched = SCHED_RTC;
@@ -697,12 +713,11 @@ void player_start(struct player *player, unsigned int mode, int section, int pos
   }
 
   /* Get the starting time */
-  gettimeofday(&player->playingstarted, NULL);
-  if (!cont) {
-    player->playedsofar.tv_sec = 0;
-    player->playedsofar.tv_usec = 0;
-  }
-
+  if (cont)
+    player_reset_time(player, FALSE);
+  else
+    player_reset_time(player, TRUE);
+    
   /* Create a new thread only if there isn't one already */
   if (player->thread == NULL) {
     player->thread = g_thread_create((GThreadFunc) player_thread, player, TRUE, NULL);
@@ -726,6 +741,7 @@ void player_stop(struct player *player)
     player->killthread = 1;
     g_mutex_unlock(player->mutex);
     g_thread_join(player->thread);
+
     player->thread = NULL;
     player->killthread = 0;
   } else
@@ -849,3 +865,19 @@ void player_set_line(struct player *player, int line)
   player->line = line;
   g_mutex_unlock(player->mutex);
 }
+
+/* Resets the player time */
+void player_reset_time(struct player *player, gboolean reset_sofar)
+{
+  if (player == NULL) {
+    fprintf(stderr, "player_stop() called with null player\n");
+    return;
+  }
+
+  gettimeofday(&player->playingstarted, NULL);
+
+  if (reset_sofar) {
+    player->playedsofar.tv_sec = 0;
+    player->playedsofar.tv_usec = 0;
+  }
+}
