@@ -1,6 +1,6 @@
 /* callbacks.c
  *
- * Copyright 2002-2003 Vesa Halttunen
+ * Copyright 2002-2004 Vesa Halttunen
  *
  * This file is part of Tutka. The code in this file is based on 
  * The Real SoundTracker - GTK+ Tracker widget
@@ -28,7 +28,7 @@
 #include "trackerwidget.h"
 #include "editor.h"
 
-const char *const notenames[133] = {
+const char *const notenames[128] = {
   "---",
   "C-1", "C#1", "D-1", "D#1", "E-1", "F-1", "F#1", "G-1", "G#1", "A-1", "A#1", "B-1",
   "C-2", "C#2", "D-2", "D#2", "E-2", "F-2", "F#2", "G-2", "G#2", "A-2", "A#2", "B-2",
@@ -40,7 +40,7 @@ const char *const notenames[133] = {
   "C-8", "C#8", "D-8", "D#8", "E-8", "F-8", "F#8", "G-8", "G#8", "A-8", "A#8", "B-8",
   "C-9", "C#9", "D-9", "D#9", "E-9", "F-9", "F#9", "G-9", "G#9", "A-9", "A#9", "B-9",
   "C-A", "C#A", "D-A", "D#A", "E-A", "F-A", "F#A", "G-A", "G#A", "A-A", "A#A", "B-A",
-  "C-B", "C#B", "D-B", "D#B", "E-B", "F-B", "F#B", "G-B", "G#B", "A-B", "A#B", "B-B",
+  "C-B", "C#B", "D-B", "D#B", "E-B", "F-B", "F#B"
 };
 
 static void init_display(Tracker *t, int width, int height);
